# -*- coding: utf-8 -*-
from optimus.utils.assets import synchronize_assets_sources
from optimus.utils.directory import (init_directory,
                                     recursive_directories_create, initialize)
from optimus.utils.text import UnicodeMixin
from optimus.utils.settings import display_settings
from optimus.utils.server import get_host_parts


__all__ = [
    'synchronize_assets_sources',
    'init_directory', 'recursive_directories_create', 'initialize',
    'UnicodeMixin',
    'display_settings',
    'get_host_parts',
]



# -*- coding: utf-8 -*-
"""
Project templates
=================

Available embedded project templates to use with project starter.

"""
TEMPLATE_ALIAS = {
    'basic': 'optimus.samples.basic',
    'i18n': 'optimus.samples.i18n',
}



# -*- coding: utf-8 -*-
import os
import click

from optimus.conf.loader import PROJECT_DIR_ENVVAR, SETTINGS_NAME_ENVVAR
from optimus.i18n.manager import I18NManager
from optimus.utils import display_settings


@click.command('po', short_help="Manage project translation catalogs")
@click.option('--init', is_flag=True,
              help=("Initialize structure, create template catalog (POT) and "
                    "initialize catalogs (PO)"))
@click.option('--update', is_flag=True,
              help=("Extract translations, update the template catalog (POT) "
                    "and update the catalogs (PO)"))
@click.option('--compile', is_flag=True,
              help=("Process to compilation of catalogs"))
@click.option('--basedir', metavar='PATH', type=click.Path(exists=True),
              help=("Base directory where to search for settings file. "
                    "Default value use current directory."),
              default=os.getcwd())
@click.option('--settings-name', metavar='NAME',
              help=("Settings file name to use without '.py' extension. "
                    "Default value is 'settings'."),
              default="settings")
@click.pass_context
def po_command(context, init, update, compile, basedir, settings_name):
    """
    Manage project translation catalogs for all registred languages
    """
    # Set required environment variables to load settings
    if PROJECT_DIR_ENVVAR not in os.environ \
       or not os.environ[PROJECT_DIR_ENVVAR]:
        os.environ[PROJECT_DIR_ENVVAR] = basedir
    if SETTINGS_NAME_ENVVAR not in os.environ \
       or not os.environ[SETTINGS_NAME_ENVVAR]:
        os.environ[SETTINGS_NAME_ENVVAR] = settings_name

    # Load current project settings
    from optimus.conf.registry import settings

    # Debug output
    display_settings(settings, ('DEBUG', 'PROJECT_DIR', 'SOURCES_DIR',
                                'TEMPLATES_DIR', 'LOCALES_DIR'))

    # Proceed to operations
    i18n = I18NManager(settings)

    if init or update or compile:
        i18n.init_locales_dir()
        i18n.build_pot(force=update)
        i18n.init_catalogs()

    if update:
        i18n.update_catalogs()

    if compile:
        i18n.compile_catalogs()



# -*- coding: utf-8 -*-
import os
import logging
import click

from optimus.conf.loader import PROJECT_DIR_ENVVAR, SETTINGS_NAME_ENVVAR
from optimus.exceptions import InvalidHostname
from optimus.utils import display_settings, get_host_parts


try:
    import cherrypy
except ImportError:
    CHERRYPY_AVAILABLE = False
else:
    CHERRYPY_AVAILABLE = True


@click.command('runserver', short_help=("Launch a simple HTTP server on "
                                        "built project"))
@click.argument('hostname', default="127.0.0.1:80")
@click.option('--basedir', metavar='PATH', type=click.Path(exists=True),
              help=("Base directory where to search for settings file. "
                    "Default value use current directory."),
              default=os.getcwd())
@click.option('--settings-name', metavar='NAME',
              help=("Settings file name to use without '.py' extension. "
                    "Default value is 'settings'."),
              default="settings")
@click.option('--index', metavar='FILENAME',
              help=("Filename to use as directory index. "
                    "Default value is 'index.html'."),
              default="index.html")
@click.pass_context
def runserver_command(context, basedir, settings_name, index, hostname):
    """
    Launch a simple HTTP server rooted on the project build directory

    Default behavior is to bind server on IP address '127.0.0.1' and port
    '80'. You may give another host to bind to as argument 'HOSTNAME'.

    'HOSTNAME' can be either a simple address like '0.0.0.0' or an address and
    port like '0.0.0.0:8001'. If no custom port is given, '80' is used as
    default.
    """
    logger = logging.getLogger("optimus")

    if not CHERRYPY_AVAILABLE:
        logger.error(("Unable to import CherryPy, you need to install it "
                      "with 'pip install cherrypy'"))
        raise click.Abort()

    # Set required environment variables to load settings
    if PROJECT_DIR_ENVVAR not in os.environ \
       or not os.environ[PROJECT_DIR_ENVVAR]:
        os.environ[PROJECT_DIR_ENVVAR] = basedir
    if SETTINGS_NAME_ENVVAR not in os.environ \
       or not os.environ[SETTINGS_NAME_ENVVAR]:
        os.environ[SETTINGS_NAME_ENVVAR] = settings_name

    # Load current project settings
    from optimus.conf.registry import settings

    # Debug output
    display_settings(settings, ('DEBUG', 'PROJECT_DIR', 'SOURCES_DIR',
                                'TEMPLATES_DIR', 'LOCALES_DIR'))

    # Get hostname to bind to
    try:
        address, port = get_host_parts(hostname)
    except InvalidHostname as e:
        logger.error(e)
        raise click.Abort()

    # Check project publish directory exists
    if not os.path.exists(settings.PUBLISH_DIR):
        logger.error(("Publish directory does not exist yet, you need to "
                      "build it before"))
        raise click.Abort()

    # Run server with publish directory served with tools.staticdir
    logger.info(("Running HTTP server on "
                 "address {address} with port {port}").format(
                    address=address,
                    port=port,
               ))

    # Configure webapp server
    cherrypy.config.update({
        'server.socket_host': address,
        'server.socket_port': port,
        'engine.autoreload_on': False,
    })

    # Configure webapp static
    conf = {
        '/': {
            'tools.staticdir.index': index,
            'tools.staticdir.on': True,
            'tools.staticdir.dir': settings.PUBLISH_DIR,
        },
    }

    cherrypy.quickstart(None, '/', config=conf)



# -*- coding: utf-8 -*-
import os

import click

from optimus.conf.loader import PROJECT_DIR_ENVVAR, SETTINGS_NAME_ENVVAR
from optimus.utils import initialize, display_settings
from optimus.conf.loader import import_pages_module
from optimus.pages.builder import PageBuilder
from optimus.assets.registry import register_assets


@click.command('build', short_help="Build project pages")
@click.option('--basedir', metavar='PATH', type=click.Path(exists=True),
              help=("Base directory where to search for settings file. "
                    "Default value use current directory."),
              default=os.getcwd())
@click.option('--settings-name', metavar='NAME',
              help=("Settings file name to use without '.py' extension. "
                    "Default value is 'settings'."),
              default="settings")
@click.pass_context
def build_command(context, basedir, settings_name):
    """
    Build project pages
    """
    # Set required environment variables to load settings
    if PROJECT_DIR_ENVVAR not in os.environ \
       or not os.environ[PROJECT_DIR_ENVVAR]:
        os.environ[PROJECT_DIR_ENVVAR] = basedir
    if SETTINGS_NAME_ENVVAR not in os.environ \
       or not os.environ[SETTINGS_NAME_ENVVAR]:
        os.environ[SETTINGS_NAME_ENVVAR] = settings_name

    # Load current project settings
    from optimus.conf.registry import settings

    # Debug output
    display_settings(settings, ('DEBUG', 'PROJECT_DIR', 'SOURCES_DIR',
                                'TEMPLATES_DIR', 'LOCALES_DIR'))

    initialize(settings)

    # Init webassets and builder
    assets_env = register_assets(settings)
    builder = PageBuilder(settings, assets_env=assets_env)
    pages_map = import_pages_module(settings.PAGES_MAP, basedir=basedir)

    # Proceed to page building from registered pages
    builder.build_bulk(pages_map.PAGES)



# -*- coding: utf-8 -*-
"""
Settings registration
*********************

This is the entry point to reach settings from project page module.
"""
import os

from optimus.conf.loader import (PROJECT_DIR_ENVVAR, SETTINGS_NAME_ENVVAR,
                                 import_settings)

try:
    basedir = os.environ[PROJECT_DIR_ENVVAR]
    # If it's set but is an empty string.
    if not basedir:
        raise KeyError
except KeyError:
    raise ImportError(("Settings cannot be imported, because "
                       "environment variable {} is "
                       "undefined.").format(PROJECT_DIR_ENVVAR))

try:
    name = os.environ[SETTINGS_NAME_ENVVAR]
    # If it's set but is an empty string.
    if not name:
        raise KeyError
except KeyError:
    raise ImportError(("Settings cannot be imported, because "
                       "environment variable {} is "
                       "undefined.").format(SETTINGS_NAME_ENVVAR))

# Reachable settings object
settings = import_settings(name, basedir)



# -*- coding: utf-8 -*-
"""
Project starter
===============

This component eases starting a new project without to recreate basic structure
each time.

Project structure and configuration are created using project template.
A project template contains some sources to copy in new project directory
and render some scripts templates (like settings or page views files).

Once done, the project is ready to be used.

Project template can be either an embedded one from Optimus or an external
one available as a Python package, its Python path will be used to reach it.

"""
import logging
import io
import os
import shutil

from string import Template
from importlib import import_module

from optimus.utils import (recursive_directories_create,
                           synchronize_assets_sources)
from optimus.samples import TEMPLATE_ALIAS
from optimus.exceptions import (DestinationExists, TemplateImportError,
                                TemplateSettingsInvalidError)


class ProjectStarter(object):
    """
    Object to create a new project with its settings, directory structure,
    scripts and assets.

    Keyword Arguments:
        dry_run (bool): Dry run mode to perform all tasks but never create
            anything on File System.

    Attributes:
        dry_run (bool): Dry run mode state.
        logger (logging.Logger): Application logger.
    """
    def __init__(self, dry_run=False):
        self.dry_run = dry_run
        self.logger = logging.getLogger('optimus')

    def check_destination(self, basedir, name):
        """
        Merge basedir and name into destination path then check if it does not
        allready exist.

        Arguments:
            basedir (str): Path to directory where to create new project.
            name (str): Directory name to create inside ``basedir``.

        Raises:
            optimus.exception.DestinationExists: If destination allready
                exists.

        Returns:
            string: Destination path.
        """
        dest = os.path.join(basedir, name)

        if os.path.exists(dest):
            raise DestinationExists("Destination allready "
                                    "exists : {}".format(dest))

        return dest

    def get_template_pythonpath(self, name):
        """
        Return Python path for template.

        Arguments:
            name (str): Either a full Python path to a template module or an
                alias defined from ``optimus.samples.TEMPLATE_ALIAS``.

        Returns:
            string: Template module Python path.
        """
        if name in TEMPLATE_ALIAS:
            name = TEMPLATE_ALIAS[name]
            self.logger.debug(("Resolved project template name alias "
                               "to: {}").format(name))

        return name

    def get_template_module(self, path):
        """
        Return template module if valid.

        Arguments:
            path (str): Python path to template module.

        Raises:
            optimus.exception.TemplateImportError: If template module import
                fails.

        Returns:
            string: Template module.
        """
        try:
            mod = import_module(path)
        except ImportError:
            raise TemplateImportError("There is no project template module "
                                      "named '{}'".format(path))

        return mod

    def deploy_assets(self, manifest, template_fspath, destination):
        """
        Copy directories defined in ``FILES_TO_SYNC`` from template
        manifest into created project.

        Arguments:
            manifest (object): Template manifest object.
            template_fspath (str): Template path where to get the locale
                directory.
            destination (str): Destination path (the created project
                directory).

        Returns:
            list: List of deployed asset directories.
        """
        deployed = []

        for src, dst in manifest.FILES_TO_SYNC:
            sources_from = os.path.join(template_fspath,
                                        manifest.SOURCES_FROM)

            sources_to = os.path.join(destination, manifest.SOURCES_TO)

            deployed.append(
                synchronize_assets_sources(sources_from, sources_to, src, dst,
                                           dry_run=self.dry_run)
            )

        # Drop 'None' values from unexisting files
        return filter(None, deployed)

    def deploy_language_files(self, manifest, template_fspath, destination):
        """
        Write provided scripts from template into create project

        Arguments:
            manifest (object): Template manifest object.
            template_fspath (str): Template path where to get the locale
                directory.
            destination (str): Destination path (the created project
                directory).
        """
        locale_src = os.path.join(template_fspath, manifest.LOCALE_DIR)
        locale_dst = os.path.join(destination, manifest.LOCALE_DIR)

        self.logger.info("Installing messages catalogs")

        if not os.path.exists(locale_src):
            msg = "Message catalog directory does not exists: {}"
            raise TemplateSettingsInvalidError(msg.format(locale_src))

        if not self.dry_run:
            shutil.copytree(locale_src, locale_dst)

    def deploy_scripts(self, manifest, source_path, destination, context={}):
        """
        Write provided scripts from project template into created project.

        Arguments:
            manifest (object): Template manifest object.
            source_path (str): Path to directory containing script sources.
            destination (str): Destination path (the created project
                directory).

        Keyword Arguments:
            context (dict): Context of variables to give to script template to
                render. Default to a empty dict.

        Returns:
            list: List of deployed asset directories.
        """
        deployed = []

        for item in manifest.SCRIPT_FILES:
            src = os.path.join(source_path, item[0])
            dst = os.path.join(destination, item[1])
            self.logger.info("* Installing '{}' to '{}'".format(src, dst))
            self.render_script(src, dst, context=context)
            deployed.append(dst)

        return deployed

    def render_script(self, filepath, destination, context={}):
        """
        Render script source into created project using ``string.Template``.

        Arguments:
            filepath (str): Filepath source.
            destination (str): Filepath destination.

        Keyword Arguments:
            context (dict): Context of variables to give to script template to
                render. Default to a empty dict.
        """
        # reading template file
        with io.open(filepath, 'r', encoding='utf-8') as f:
            content = Template(f.read())
        # render content
        content = content.substitute(**context)
        self.logger.debug("  Writing")

        if not self.dry_run:
            # check file destination and creating it if needed
            dest_path = os.path.dirname(destination)
            if not os.path.exists(dest_path):
                os.makedirs(dest_path)
            # writing file
            with io.open(destination, 'w', encoding='utf-8') as f:
                f.write(content)

    def install(self, basedir, name, template_pythonpath):
        """
        Install new project structure and content from project template.

        Arguments:
            basedir (str): Path to the directory where to create new project.
            name (str): Name of the new project, will be also the dir name of
                the created project, this must be a valid module name (without
                spaces, special chars, etc..)
            template_pythonpath (str): Python path or alias name to the
                template module.

        Returns:
            string: Path where the new project has been created.
        """
        destination = self.check_destination(basedir, name)

        template_pythonpath = self.get_template_pythonpath(template_pythonpath)

        self.logger.info("Loading project template "
                         "from : {}".format(template_pythonpath))

        manifest = self.get_template_module(template_pythonpath)

        template_fspath = os.path.abspath(os.path.dirname(manifest.__file__))

        self.logger.info("Creating new Optimus project '{}' "
                         "in : {}".format(name, basedir))
        if not self.dry_run:
            os.makedirs(destination)

        self.logger.info("Installing directories structure "
                         "to : {}".format(destination))
        recursive_directories_create(destination,
                                     manifest.DIRECTORY_STRUCTURE,
                                     dry_run=self.dry_run)

        self.logger.info("Copying templates sources "
                         "to: {}".format(destination))
        self.deploy_assets(manifest, template_fspath, destination)

        if hasattr(manifest, "LOCALE_DIR"):
            self.deploy_language_files(manifest, template_fspath, destination)

        self.logger.info("Installing default project's files")
        context = {
            'PROJECT_NAME': name,
            'SOURCES_FROM': manifest.SOURCES_FROM,
        }
        self.logger.debug("Getting files from '{}'".format(template_fspath))
        self.deploy_scripts(manifest, template_fspath, destination, context)

        return destination



# -*- coding: utf-8 -*-
"""
Module loader helpers
*********************

TODO:
    * 'imp' is deprecated since python ~3.4 => To remove in favor of importlib
      or make a switch depending of python version;
"""
import os
import imp
import logging
import sys

from optimus.conf.model import SettingsModel


PROJECT_DIR_ENVVAR = "OPTIMUS_PROJECT_DIR"

SETTINGS_NAME_ENVVAR = "OPTIMUS_SETTINGS_MODULE"


def import_project_module(name, basedir=None,
                          finding_module_err='Unable to find module: {0}',
                          import_module_err='Unable to load module: {0}'):
    """
    Load given module name.

    Arguments:
        name (str): Module name to retrieve from ``basedir``.

    Keyword Arguments:
        basedir (str): Base directory from where to find module name. If no
            base directory is given ``os.getcwd()`` is used. Default is
            ``None``.
        finding_module_err (str): Message to output when the given module name
            is not reachable from ``basedir``.
        import_module_err (str): Message to output when the given module name
            raise exception when loaded.

    Returns:
        object: Finded and loaded module.
    """
    basedir = basedir or os.getcwd()

    logger = logging.getLogger('optimus')
    logger.info('Loading "%s" module', name)
    logger.info('Module searched in: %s', basedir)

    # Add the project to the sys.path
    project_name = os.path.basename(os.path.abspath(basedir))
    sys.path.append(os.path.normpath(os.path.join(basedir, '..')))
    # Sys.path is ok, we can import the project
    try:
        __import__(project_name, '', '', [''])
    except ImportError:
        msg = "Unable to load project named: {0}"
        logger.critical(msg.format(project_name))
        raise
    # Cleanup the sys.path of the project path
    sys.path.pop()

    fp = pathname = description = None
    try:
        fp, pathname, description = imp.find_module(name, [basedir])
    except ImportError:
        logger.critical(finding_module_err.format(name))
        # dont raising exception that is not really helping since it point out
        # to 'imp.find_module' line
        sys.exit()
    else:
        try:
            mod = imp.load_module(name, fp, pathname, description)
        except: # noqa
            logger.critical(import_module_err.format(name))
            # Print out the exception because it is very useful to debug
            raise
            sys.exit()
    finally:
        # Close fp explicitly.
        if fp:
            fp.close()

    return mod


def import_settings_module(name, basedir=None):
    """
    Shortcut to have specific error message when loading settings module

    Arguments:
        name (str): Module name to retrieve from ``basedir``.

    Keyword Arguments:
        basedir (str): Base directory from where to find module name. If no
            base directory is given ``os.getcwd()`` is used. Default is
            ``None``.

    Returns:
        object: Finded and loaded module.
    """
    msg_finding = "Unable to find settings module: {0}"
    msg_import = "Unable to load settings module, it probably have errors: {0}"
    return import_project_module(name, basedir=basedir,
                                 finding_module_err=msg_finding,
                                 import_module_err=msg_import)


def import_pages_module(name, basedir=None):
    """
    Shortcut to have specific error message when loading a page module

    Arguments:
        name (str): Module name to retrieve from ``basedir``.

    Keyword Arguments:
        basedir (str): Base directory from where to find module name. If no
            base directory is given ``os.getcwd()`` is used. Default is
            ``None``.

    Returns: Finded and loaded module.
    """
    msg_finding = "Unable to find pages module: {0}"
    msg_import = "Unable to load pages module, it probably have errors: {0}"
    return import_project_module(name, basedir=basedir,
                                 finding_module_err=msg_finding,
                                 import_module_err=msg_import)


def import_settings(name, basedir):
    """
    Load settings module.

    Validate required settings are set, then fill some missing settings to a
    default value.

    Arguments:
        name (str): Settings module name to retrieve from ``basedir``.
        basedir (str): Base directory from where to find settings module name.

    Returns:
        object: Settings module.
    """
    settings_module = import_settings_module(name, basedir)

    _settings = SettingsModel()
    _settings.load_from_module(settings_module)

    return _settings



# -*- coding: utf-8 -*-
# flake8: noqa
#import copy

#import docutils
#import docutils.core
#import docutils.nodes
#import docutils.utils
#import docutils.parsers.rst

#from optimus.html5writer import SemanticHTML5Writer


#class RstPageView(PageViewBase):
    #"""
    #View to build a page from a ReStructuredText file

    #You need to set the ``source_filepath`` class attributes in addition to the required
    #ones from ``PageItemBase``. ``parser_settings`` is an optionnal class attribute as a
    #dict that is passed to the docutils parser. If not given, it will be filled from the
    #``RST_PARSER_SETTINGS`` settings option if not empty.

    #Two additionals variables will be added to the context :

    #* page_doc_html: the HTML produced by the parser from the rst document
    #* page_doc_source: the unparsed source from the rst document

    #This is almost deprecated and has not test coverage.
    #"""
    #source_filepath = None
    #parser_settings = {}

    #def get_source_filepath(self):
        #return self.source_filepath

    #def get_context(self):
        #context = super(RstPageView, self).get_context()

        #rst_parser_settings = copy.deepcopy(getattr(self.settings, 'RST_PARSER_SETTINGS', {}))
        #rst_parser_settings.update(self.parser_settings)

        #f = open(self.get_source_filepath(), 'r')
        #doc_source = f.read()
        #f.close()
        #parts = docutils.core.publish_parts(source=doc_source, writer=SemanticHTML5Writer(), settings_overrides=rst_parser_settings)

        #context.update({
            #'page_doc_html': parts['fragment'],
            #'page_doc_source': doc_source,
        #})
        #return context



# -*- coding: utf-8 -*-
import os
import logging
import time

import click

from watchdog.observers import Observer

from optimus.conf.loader import PROJECT_DIR_ENVVAR, SETTINGS_NAME_ENVVAR
from optimus.utils import initialize, display_settings
from optimus.conf.loader import import_pages_module
from optimus.pages.builder import PageBuilder
from optimus.assets.registry import register_assets
from optimus.watchers.templates import TemplatesWatchEventHandler
from optimus.watchers.assets import AssetsWatchEventHandler


@click.command('watch', short_help="Watch for changes in project sources")
@click.option('--basedir', metavar='PATH', type=click.Path(exists=True),
              help=("Base directory where to search for settings file. "
                    "Default value use current directory."),
              default=os.getcwd())
@click.option('--settings-name', metavar='NAME',
              help=("Settings file name to use without '.py' extension. "
                    "Default value is 'settings'."),
              default="settings")
@click.pass_context
def watch_command(context, basedir, settings_name):
    """
    Watch for changes in project sources to automatically build project
    ressources
    """
    logger = logging.getLogger("optimus")

    # Set required environment variables to load settings
    if PROJECT_DIR_ENVVAR not in os.environ \
       or not os.environ[PROJECT_DIR_ENVVAR]:
        os.environ[PROJECT_DIR_ENVVAR] = basedir
    if SETTINGS_NAME_ENVVAR not in os.environ or \
       not os.environ[SETTINGS_NAME_ENVVAR]:
        os.environ[SETTINGS_NAME_ENVVAR] = settings_name

    # Load current project settings
    from optimus.conf.registry import settings

    # Debug output
    display_settings(settings, ('DEBUG', 'PROJECT_DIR', 'SOURCES_DIR',
                                'TEMPLATES_DIR', 'LOCALES_DIR'))

    initialize(settings)

    # Init webassets and builder
    assets_env = register_assets(settings)
    builder = PageBuilder(settings, assets_env=assets_env)
    pages_map = import_pages_module(settings.PAGES_MAP, basedir=basedir)

    # Proceed to page building from registered pages
    logger.debug('Trigger pages build to start')
    builder.build_bulk(pages_map.PAGES)

    builder.scan_bulk(pages_map.PAGES)

    observer = Observer()

    # Init templates and assets event watchers
    templates_event_handler = TemplatesWatchEventHandler(
        settings,
        builder,
        **settings.WATCHER_TEMPLATES_PATTERNS
    )

    if assets_env is not None:
        assets_event_handler = AssetsWatchEventHandler(
            settings,
            assets_env,
            builder,
            **settings.WATCHER_ASSETS_PATTERNS
        )

    # Registering event watchers and start to watch
    observer.schedule(
        templates_event_handler,
        settings.TEMPLATES_DIR,
        recursive=True
    )
    if assets_env is not None:
        observer.schedule(
            assets_event_handler,
            settings.SOURCES_DIR,
            recursive=True
        )

    logger.warning('Starting to watch sources, use CTRL+C to stop it')
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        logger.warning('Stopping watcher..')
        observer.stop()
    observer.join()



# -*- coding: utf-8 -*-
import os
import logging
import click

from string import ascii_letters, digits

from optimus.start_project import ProjectStarter


@click.command('init', short_help="Create a new project from a template")
@click.argument('name')
@click.option('--template', metavar='NAME',
              help=(("A template name either 'basic' or 'i18n'. Also a valid "
                     "Python path to an external template can be given."
                     "Default value is 'basic'.")),
              default="basic")
@click.option('--dry-run', is_flag=True,
              help=("Dry run mode will perform all processus but will not "
                    "create or modify anything"))
@click.pass_context
def startproject_command(context, name, template, dry_run):
    """
    Create a new project from a template

    Attempt one argument 'NAME' that will be the project name and its directory
    name. The name must be a valid Python module name.
    """
    logger = logging.getLogger("optimus")

    # Valid that all characters from the name are : "_" character, letters,
    # identifier ::=  (letter|"_") (letter | digit | "_")*
    # This is not fully safe, user can create a project name using an installed
    # Python module that will override it and make some troubles in some case
    if name:
        if name[0] not in ascii_letters:
            logger.error("Project name must start with a letter")
            return
        for k in name[1:]:
            if k not in ascii_letters and k not in digits and k != "_":
                logger.error(("Project name must only contains letters, "
                              "digits or '_' character"))
                return

    if dry_run:
        logger.warning("'Dry run mode enabled")

    # TODO: optionnal command option to specify another path where the project
    #       will be created
    project_directory = os.path.abspath(os.getcwd())

    starter = ProjectStarter(dry_run=dry_run)
    starter.install(project_directory, name, template)



# flake8: noqa
"""
Original code is from 'Bradley Wright' on Github :

https://github.com/bradleywright/rst-to-semantic-html5

Stealed and modified as i want to keep <section/> and i don't want to take the same way
than 'rst-to-semantic-html5'
"""
#import re

#from docutils import nodes, utils
#from docutils.writers import html4css1
#from docutils.core import publish_parts
#from docutils.parsers.rst import roles, directives, Directive


#class SemanticHTML5Writer(html4css1.Writer):
    #"""
    #This docutils writer will use the SemanticHTML5Translator class below.
    #"""
    #def __init__(self):
        #html4css1.Writer.__init__(self)
        #self.translator_class = SemanticHTML5Translator


#class SemanticHTML5Translator(html4css1.HTMLTranslator):
    #"""
    #This is a translator class for the docutils system.
    #It will produce a minimal set of html output.
    #(No extra divs, classes over ids.)

    #It also aims to produce HTML5 (section etc.)
    #"""
    #def should_be_compact_paragraph(self, node):
        #if(isinstance(node.parent, nodes.block_quote)):
            #return 0
        #return html4css1.HTMLTranslator.should_be_compact_paragraph(self, node)

    #def visit_section(self, node):
        #self.section_level += 1
        #self.body.append(self.starttag(node, 'section', ''))

    #def depart_section(self, node):
        #self.section_level -= 1
        #self.body.append('</section>')

    #def visit_literal_block(self, node):
        #"No classes please"
        #self.body.append(self.starttag(node, 'pre', ''))

    #def visit_literal(self, node):
        #self.body.append(
            #self.starttag(node, 'code', ''))

    #def depart_literal(self, node):
        #self.body.append('</code>')

    #def visit_abbreviation(self, node):
        #attrs = {}
        #if node.hasattr('explanation'):
            #attrs['title'] = node['explanation']
        #self.body.append(self.starttag(node, 'abbr', '', **attrs))

    #def depart_abbreviation(self, node):
        #self.body.append('</abbr>')

    #def visit_kbd(self, node):
        #self.body.append(self.starttag(node, 'kbd', ''))

    #def depart_kbd(self, node):
        #self.body.append('</kbd>')


#class kbd(nodes.Inline, nodes.TextElement):
    #"""Node for kbd element"""

#nodes._add_node_class_names('kbd')


#def inline_roles(role, raw, text, *args):
    #if role == 'kbd':
        #return [kbd('kbd', text)], []
    #elif role == 'var':
        #return [nodes.literal('var', text)], []


#roles.register_local_role('kbd', inline_roles)
#roles.register_local_role('var', inline_roles)


## FIXME: this has to be lowercase for some reason
#class abbreviation(nodes.Inline, nodes.TextElement):
    #"""Node for abbreviations with explanations."""


#nodes._add_node_class_names('abbreviation')


#_abbr_re = re.compile('\((.*)\)$', re.S)


#def abbr_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
    #text = utils.unescape(text)
    #m = _abbr_re.search(text)
    #if m is None:
        #return [abbreviation(text, text)], []
    #abbr = text[:m.start()].strip()
    #expl = m.group(1)
    #return [abbreviation(abbr, abbr, explanation=expl)], []

#roles.register_local_role('abbr', abbr_role)



# -*- coding: utf-8 -*-
import sys


class UnicodeMixin(object):
    """
    Helper class to ensure ``_str__`` compatibility for python 2 and 3.
    """
    if sys.version_info > (3, 0):
        __str__ = lambda x: x.__unicode__() # noqa
    else:
        __str__ = lambda x: unicode(x).encode('utf-8') # noqa
